@@ -1,21 +1,30 @@
 """A collection of CLI commands for working with Kedro micro-packages."""
 
+# ruff: noqa: I001
+from __future__ import annotations
+
+import logging
 import re
 import shutil
 import sys
 import tarfile
 import tempfile
+import toml
 from importlib import import_module
 from pathlib import Path
-from typing import Iterable, List, Optional, Set, Tuple, Union
+from typing import Any, TYPE_CHECKING
+
 
 import click
-import pkg_resources
+from omegaconf import OmegaConf
+from packaging.requirements import InvalidRequirement, Requirement
+from packaging.utils import canonicalize_name
 from rope.base.project import Project
 from rope.contrib import generate
 from rope.refactor.move import MoveModule
 from rope.refactor.rename import Rename
 
+from build.util import project_wheel_metadata
 from kedro.framework.cli.pipeline import (
     _assert_pkg_name_ok,
     _check_pipeline_name,
@@ -30,23 +39,80 @@ from kedro.framework.cli.utils import (
     env_option,
     python_call,
 )
-from kedro.framework.startup import ProjectMetadata
 
-_SETUP_PY_TEMPLATE = """# -*- coding: utf-8 -*-
-from setuptools import setup, find_packages
+if TYPE_CHECKING:
+    from kedro.framework.startup import ProjectMetadata
+    from importlib_metadata import PackageMetadata
+    from collections.abc import Iterable, Iterator
 
-setup(
-    name="{name}",
-    version="{version}",
-    description="Micro-package `{name}`",
-    packages=find_packages(),
-    include_package_data=True,
-    install_requires={install_requires},
-)
+_PYPROJECT_TOML_TEMPLATE = """
+[build-system]
+requires = ["setuptools"]
+build-backend = "setuptools.build_meta"
+
+[project]
+name = "{name}"
+version = "{version}"
+description = "Micro-package `{name}`"
+dependencies = {install_requires}
+
+[tool.setuptools.packages]
+find = {{}}
 """
 
+logger = logging.getLogger(__name__)
 
-def _check_module_path(ctx, param, value):  # pylint: disable=unused-argument
+
+class _EquivalentRequirement(Requirement):
+    """Parse a requirement according to PEP 508.
+
+    This class overrides __eq__ to be backwards compatible with pkg_resources.Requirement
+    while making __str__ and __hash__ use the non-canonicalized name
+    as agreed in https://github.com/pypa/packaging/issues/644,
+
+    Implementation taken from https://github.com/pypa/packaging/pull/696/
+    """
+
+    def _iter_parts(self, name: str) -> Iterator[str]:
+        yield name
+
+        if self.extras:
+            formatted_extras = ",".join(sorted(self.extras))
+            yield f"[{formatted_extras}]"
+
+        if self.specifier:
+            yield str(self.specifier)
+
+        if self.url:
+            yield f"@ {self.url}"
+            if self.marker:
+                yield " "
+
+        if self.marker:
+            yield f"; {self.marker}"
+
+    def __str__(self) -> str:
+        return "".join(self._iter_parts(self.name))
+
+    def __hash__(self) -> int:
+        return hash(
+            (
+                self.__class__.__name__,
+                *self._iter_parts(canonicalize_name(self.name)),
+            )
+        )
+
+    def __eq__(self, other: Any) -> bool:
+        return (
+            canonicalize_name(self.name) == canonicalize_name(other.name)
+            and self.extras == other.extras
+            and self.specifier == other.specifier
+            and self.url == other.url
+            and self.marker == other.marker
+        )
+
+
+def _check_module_path(ctx: click.core.Context, param: Any, value: str) -> str:
     if value and not re.match(r"^[\w.]+$", value):
         message = (
             "The micro-package location you provided is not a valid Python module path"
@@ -55,15 +121,15 @@ def _check_module_path(ctx, param, value):  # pylint: disable=unused-argument
     return value
 
 
-# pylint: disable=missing-function-docstring
 @click.group(name="Kedro")
-def micropkg_cli():  # pragma: no cover
+def micropkg_cli() -> None:  # pragma: no cover
     pass
 
 
 @micropkg_cli.group()
-def micropkg():
-    """Commands for working with micro-packages."""
+def micropkg() -> None:
+    """(DEPRECATED) Commands for working with micro-packages. DeprecationWarning: micro-packaging is deprecated
+    and will not be available from Kedro 1.0.0."""
 
 
 @command_with_verbosity(micropkg, "pull")
@@ -95,17 +161,24 @@ def micropkg():
     help="Location of a configuration file for the fsspec filesystem used to pull the package.",
 )
 @click.pass_obj  # this will pass the metadata as first argument
-def pull_package(  # pylint:disable=unused-argument, too-many-arguments
+def pull_package(  # noqa: PLR0913
     metadata: ProjectMetadata,
-    package_path,
-    env,
-    alias,
-    destination,
-    fs_args,
-    all_flag,
-    **kwargs,
+    /,
+    package_path: str,
+    env: str,
+    alias: str,
+    destination: str,
+    fs_args: str,
+    all_flag: str,
+    **kwargs: Any,
 ) -> None:
-    """Pull and unpack a modular pipeline and other micro-packages in your project."""
+    """(DEPRECATED) Pull and unpack a modular pipeline and other micro-packages in your project."""
+    deprecation_message = (
+        "DeprecationWarning: Command 'kedro micropkg pull' is deprecated and "
+        "will not be available from Kedro 1.0.0."
+    )
+    click.secho(deprecation_message, fg="red")
+
     if not package_path and not all_flag:
         click.secho(
             "Please specify a package path or add '--all' to pull all micro-packages in the "
@@ -130,48 +203,71 @@ def pull_package(  # pylint:disable=unused-argument, too-many-arguments
     click.secho(message, fg="green")
 
 
-# pylint: disable=too-many-arguments, too-many-locals
-def _pull_package(
+def _pull_package(  # noqa: PLR0913
     package_path: str,
     metadata: ProjectMetadata,
-    env: str = None,
-    alias: str = None,
-    destination: str = None,
-    fs_args: str = None,
-):
+    env: str | None = None,
+    alias: str | None = None,
+    destination: str | None = None,
+    fs_args: str | None = None,
+) -> None:
     with tempfile.TemporaryDirectory() as temp_dir:
         temp_dir_path = Path(temp_dir).resolve()
         _unpack_sdist(package_path, temp_dir_path, fs_args)
 
-        egg_info_files = list((temp_dir_path).rglob("*.egg-info"))
-        if len(egg_info_files) != 1:
+        # temp_dir_path is the parent directory of the project root dir
+        contents = [member for member in temp_dir_path.iterdir() if member.is_dir()]
+        if len(contents) != 1:
             raise KedroCliError(
-                f"More than 1 or no egg-info files found from {package_path}. "
-                f"There has to be exactly one egg-info directory."
+                "Invalid sdist was extracted: exactly one directory was expected, "
+                f"got {contents}"
             )
-        egg_info_file = egg_info_files[0]
-        package_name = egg_info_file.stem
-        package_requirements = egg_info_file.parent / "setup.py"
+        project_root_dir = contents[0]
 
-        # Finds a string representation of 'install_requires' list from setup.py
-        reqs_list_pattern = r"install_requires\=(.*?)\,\n"
-        list_reqs = re.findall(
-            reqs_list_pattern, package_requirements.read_text(encoding="utf-8")
-        )
+        # This is much slower than parsing the requirements
+        # directly from the metadata files
+        # because it installs the package in an isolated environment,
+        # but it's the only reliable way of doing it
+        # without making assumptions on the project metadata.
+        library_meta = project_wheel_metadata(project_root_dir)
 
-        # Finds all elements from the above string representation of a list
-        reqs_element_pattern = r"\'(.*?)\'"
-        package_reqs = re.findall(reqs_element_pattern, list_reqs[0])
+        # Project name will be `my-pipeline` even if `pyproject.toml` says `my_pipeline`
+        # because standards mandate normalization of names for comparison,
+        # see https://packaging.python.org/en/latest/specifications/core-metadata/#name
+        # The proper way to get it would be
+        # project_name = library_meta.get("Name")
+        # However, the rest of the code expects the non-normalized package name,
+        # so we have to find it.
+        packages = [
+            project_item.name
+            for project_item in project_root_dir.iterdir()
+            if project_item.is_dir()
+            and project_item.name != "tests"
+            and (project_item / "__init__.py").exists()
+        ]
+        if len(packages) != 1:
+            # Should not happen if user is calling `micropkg pull`
+            # with the result of a `micropkg package`,
+            # and in general if the distribution only contains one package (most likely),
+            # but helps give a sensible error message otherwise
+            raise KedroCliError(
+                "Invalid package contents: exactly one package was expected, "
+                f"got {packages}"
+            )
+        package_name = packages[0]
+
+        # Type ignored because of https://github.com/pypa/build/pull/693
+        package_reqs = _get_all_library_reqs(library_meta)  # type: ignore[arg-type]
 
         if package_reqs:
-            requirements_txt = metadata.source_dir / "requirements.txt"
+            requirements_txt = metadata.project_path / "requirements.txt"
             _append_package_reqs(requirements_txt, package_reqs, package_name)
 
         _clean_pycache(temp_dir_path)
         _install_files(
             metadata,
             package_name,
-            egg_info_file.parent,
+            project_root_dir,
             env,
             alias,
             destination,
@@ -179,10 +275,7 @@ def _pull_package(
 
 
 def _pull_packages_from_manifest(metadata: ProjectMetadata) -> None:
-    # pylint: disable=import-outside-toplevel
-    import anyconfig  # for performance reasons
-
-    config_dict = anyconfig.load(metadata.config_file)
+    config_dict = toml.load(metadata.config_file)
     config_dict = config_dict["tool"]["kedro"]
     build_specs = config_dict.get("micropkg", {}).get("pull")
 
@@ -203,10 +296,7 @@ def _pull_packages_from_manifest(metadata: ProjectMetadata) -> None:
 
 
 def _package_micropkgs_from_manifest(metadata: ProjectMetadata) -> None:
-    # pylint: disable=import-outside-toplevel
-    import anyconfig  # for performance reasons
-
-    config_dict = anyconfig.load(metadata.config_file)
+    config_dict = toml.load(metadata.config_file)
     config_dict = config_dict["tool"]["kedro"]
     build_specs = config_dict.get("micropkg", {}).get("package")
 
@@ -226,7 +316,7 @@ def _package_micropkgs_from_manifest(metadata: ProjectMetadata) -> None:
     click.secho("Micro-packages packaged!", fg="green")
 
 
-@micropkg.command("package")
+@command_with_verbosity(micropkg, "package")
 @env_option(
     help="Environment where the micro-package configuration lives. Defaults to `base`."
 )
@@ -252,10 +342,23 @@ def _package_micropkgs_from_manifest(metadata: ProjectMetadata) -> None:
 )
 @click.argument("module_path", nargs=1, required=False, callback=_check_module_path)
 @click.pass_obj  # this will pass the metadata as first argument
-def package_micropkg(
-    metadata: ProjectMetadata, module_path, env, alias, destination, all_flag
-):
-    """Package up a modular pipeline or micro-package as a Python source distribution."""
+def package_micropkg(  # noqa: PLR0913
+    metadata: ProjectMetadata,
+    /,
+    module_path: str,
+    env: str,
+    alias: str,
+    destination: str,
+    all_flag: str,
+    **kwargs: Any,
+) -> None:
+    """(DEPRECATED) Package up a modular pipeline or micro-package as a Python source distribution."""
+    deprecation_message = (
+        "DeprecationWarning: Command 'kedro micropkg package' is deprecated and "
+        "will not be available from Kedro 1.0.0."
+    )
+    click.secho(deprecation_message, fg="red")
+
     if not module_path and not all_flag:
         click.secho(
             "Please specify a micro-package name or add '--all' to package all micro-packages in "
@@ -279,19 +382,17 @@ def package_micropkg(
     click.secho(message, fg="green")
 
 
-def _get_fsspec_filesystem(location: str, fs_args: Optional[str]):
-    # pylint: disable=import-outside-toplevel
-    import anyconfig
+def _get_fsspec_filesystem(location: str, fs_args: str | None) -> Any:
     import fsspec
 
     from kedro.io.core import get_protocol_and_path
 
     protocol, _ = get_protocol_and_path(location)
-    fs_args_config = anyconfig.load(fs_args) if fs_args else {}
+    fs_args_config = OmegaConf.to_container(OmegaConf.load(fs_args)) if fs_args else {}
 
     try:
         return fsspec.filesystem(protocol, **fs_args_config)
-    except Exception as exc:  # pylint: disable=broad-except
+    except Exception as exc:
         # Specified protocol is not supported by `fsspec`
         # or requires extra dependencies
         click.secho(str(exc), fg="red")
@@ -299,22 +400,25 @@ def _get_fsspec_filesystem(location: str, fs_args: Optional[str]):
         return None
 
 
-def _is_within_directory(directory, target):
+def _is_within_directory(directory: Path, target: Path) -> bool:
     abs_directory = directory.resolve()
     abs_target = target.resolve()
     return abs_directory in abs_target.parents
 
 
-def safe_extract(tar, path):
+def safe_extract(tar: tarfile.TarFile, path: Path) -> None:
+    safe_members = []
     for member in tar.getmembers():
         member_path = path / member.name
         if not _is_within_directory(path, member_path):
-            # pylint: disable=broad-exception-raised
             raise Exception("Failed to safely extract tar file.")
-    tar.extractall(path)  # nosec B202
+        safe_members.append(member)
+    tar.extractall(path, members=safe_members)  # noqa S202
+    # The nosec is still required because bandit still flags this.
+    # Related issue: https://github.com/PyCQA/bandit/issues/1038
 
 
-def _unpack_sdist(location: str, destination: Path, fs_args: Optional[str]) -> None:
+def _unpack_sdist(location: str, destination: Path, fs_args: str | None) -> None:
     filesystem = _get_fsspec_filesystem(location, fs_args)
 
     if location.endswith(".tar.gz") and filesystem and filesystem.exists(location):
@@ -323,11 +427,20 @@ def _unpack_sdist(location: str, destination: Path, fs_args: Optional[str]) -> N
                 safe_extract(tar_file, destination)
     else:
         python_call(
-            "pip", ["download", "--no-deps", "--dest", str(destination), location]
+            "pip",
+            [
+                "download",
+                "--no-deps",
+                "--no-binary",  # the micropackaging expects an sdist,
+                ":all:",  # wheels are not supported
+                "--dest",
+                str(destination),
+                location,
+            ],
         )
         sdist_file = list(destination.glob("*.tar.gz"))
-        # `--no-deps` should fetch only one source distribution file, and CLI should fail if that's
-        # not the case.
+        # `--no-deps --no-binary :all:` should fetch only one source distribution file,
+        # and CLI should fail if that's not the case.
         if len(sdist_file) != 1:
             file_names = [sf.name for sf in sdist_file]
             raise KedroCliError(
@@ -338,7 +451,7 @@ def _unpack_sdist(location: str, destination: Path, fs_args: Optional[str]) -> N
             safe_extract(fs_file, destination)
 
 
-def _rename_files(conf_source: Path, old_name: str, new_name: str):
+def _rename_files(conf_source: Path, old_name: str, new_name: str) -> None:
     config_files_to_rename = (
         each
         for each in conf_source.rglob("*")
@@ -349,14 +462,14 @@ def _rename_files(conf_source: Path, old_name: str, new_name: str):
         config_file.rename(config_file.parent / new_config_name)
 
 
-def _refactor_code_for_unpacking(
+def _refactor_code_for_unpacking(  # noqa: PLR0913
     project: Project,
     package_path: Path,
     tests_path: Path,
-    alias: Optional[str],
-    destination: Optional[str],
+    alias: str | None,
+    destination: str | None,
     project_metadata: ProjectMetadata,
-) -> Tuple[Path, Path]:
+) -> tuple[Path, Path]:
     """This is the reverse operation of `_refactor_code_for_package`, i.e
     we go from:
     <temp_dir>  # also the root of the Rope project
@@ -383,7 +496,7 @@ def _refactor_code_for_unpacking(
     """
 
     def _move_package_with_conflicting_name(
-        target: Path, original_name: str, desired_name: str = None
+        target: Path, original_name: str, desired_name: str | None = None
     ) -> Path:
         _rename_package(project, original_name, "tmp_name")
         full_path = _create_nested_package(project, target)
@@ -430,14 +543,14 @@ def _refactor_code_for_unpacking(
     return refactored_package_path, refactored_tests_path
 
 
-def _install_files(  # pylint: disable=too-many-arguments, too-many-locals
+def _install_files(  # noqa: PLR0913, too-many-locals
     project_metadata: ProjectMetadata,
     package_name: str,
     source_path: Path,
-    env: str = None,
-    alias: str = None,
-    destination: str = None,
-):
+    env: str | None = None,
+    alias: str | None = None,
+    destination: str | None = None,
+) -> None:
     env = env or "base"
 
     package_source, test_source, conf_source = _get_package_artifacts(
@@ -478,9 +591,9 @@ def _install_files(  # pylint: disable=too-many-arguments, too-many-locals
 
 
 def _find_config_files(
-    source_config_dir: Path, glob_patterns: List[str]
-) -> List[Tuple[Path, str]]:
-    config_files = []  # type: List[Tuple[Path, str]]
+    source_config_dir: Path, glob_patterns: list[str]
+) -> list[tuple[Path, str]]:
+    config_files: list[tuple[Path, str]] = []
 
     if source_config_dir.is_dir():
         config_files = [
@@ -499,19 +612,25 @@ def _get_default_version(metadata: ProjectMetadata, micropkg_module_path: str) -
         micropkg_module = import_module(
             f"{metadata.package_name}.{micropkg_module_path}"
         )
-        return micropkg_module.__version__  # type: ignore
+        return micropkg_module.__version__  # type: ignore[no-any-return]
     except (AttributeError, ModuleNotFoundError):
+        logger.warning(
+            "Micropackage version not found in '%s.%s', will take the top-level one in '%s'",
+            metadata.package_name,
+            micropkg_module_path,
+            metadata.package_name,
+        )
         # if micropkg version doesn't exist, take the project one
         project_module = import_module(f"{metadata.package_name}")
-        return project_module.__version__  # type: ignore
+        return project_module.__version__  # type: ignore[no-any-return]
 
 
 def _package_micropkg(
     micropkg_module_path: str,
     metadata: ProjectMetadata,
-    alias: str = None,
-    destination: str = None,
-    env: str = None,
+    alias: str | None = None,
+    destination: str | None = None,
+    env: str | None = None,
 ) -> Path:
     micropkg_name = micropkg_module_path.split(".")[-1]
     package_dir = metadata.source_dir / metadata.package_name
@@ -522,9 +641,16 @@ def _package_micropkg(
     )
     # as the source distribution will only contain parameters, we aren't listing other
     # config files not to confuse users and avoid useless file copies
+    # collect configs to package not only from parameters folder, but from core conf folder also
+    # because parameters had been moved from foldername to yml filename
     configs_to_package = _find_config_files(
         package_conf,
-        [f"parameters*/**/{micropkg_name}.yml", f"parameters*/**/{micropkg_name}/**/*"],
+        [
+            f"**/parameters_{micropkg_name}.yml",
+            f"**/{micropkg_name}/**/*",
+            f"parameters*/**/{micropkg_name}.yml",
+            f"parameters*/**/{micropkg_name}/**/*",
+        ],
     )
 
     source_paths = (package_source, package_tests, configs_to_package)
@@ -532,12 +658,14 @@ def _package_micropkg(
     # Check that micropkg directory exists and not empty
     _validate_dir(package_source)
 
-    destination = Path(destination) if destination else metadata.project_path / "dist"
+    package_destination = (
+        Path(destination) if destination else metadata.project_path / "dist"
+    )
     version = _get_default_version(metadata, micropkg_module_path)
 
     _generate_sdist_file(
         micropkg_name=micropkg_name,
-        destination=destination.resolve(),
+        destination=package_destination.resolve(),
         source_paths=source_paths,
         version=version,
         metadata=metadata,
@@ -547,7 +675,7 @@ def _package_micropkg(
     _clean_pycache(package_dir)
     _clean_pycache(metadata.project_path)
 
-    return destination
+    return package_destination
 
 
 def _validate_dir(path: Path) -> None:
@@ -557,23 +685,33 @@ def _validate_dir(path: Path) -> None:
         raise KedroCliError(f"'{path}' is an empty directory.")
 
 
-def _get_sdist_name(name, version):
+def _get_sdist_name(name: str, version: str) -> str:
     return f"{name}-{version}.tar.gz"
 
 
-def _sync_path_list(source: List[Tuple[Path, str]], target: Path) -> None:
+def _sync_path_list(source: list[tuple[Path, str]], target: Path) -> None:
     for source_path, suffix in source:
         target_with_suffix = (target / suffix).resolve()
         _sync_dirs(source_path, target_with_suffix)
 
 
-def _make_install_requires(requirements_txt: Path) -> List[str]:
+def _drop_comment(line: str) -> str:
+    # https://github.com/pypa/setuptools/blob/b545fc7/\
+    # pkg_resources/_vendor/jaraco/text/__init__.py#L554-L566
+    return line.partition(" #")[0]
+
+
+def _make_install_requires(requirements_txt: Path) -> list[str]:
     """Parses each line of requirements.txt into a version specifier valid to put in
-    install_requires."""
+    install_requires.
+    Matches pkg_resources.parse_requirements"""
     if not requirements_txt.exists():
         return []
-    requirements = pkg_resources.parse_requirements(requirements_txt.read_text())
-    return [str(requirement) for requirement in requirements]
+    return [
+        str(_EquivalentRequirement(_drop_comment(requirement_line)))
+        for requirement_line in requirements_txt.read_text().splitlines()
+        if requirement_line and not requirement_line.startswith("#")
+    ]
 
 
 def _create_nested_package(project: Project, package_path: Path) -> Path:
@@ -628,7 +766,7 @@ def _refactor_code_for_package(
     project: Project,
     package_path: Path,
     tests_path: Path,
-    alias: Optional[str],
+    alias: str | None,
     project_metadata: ProjectMetadata,
 ) -> None:
     """In order to refactor the imports properly, we need to recreate
@@ -664,7 +802,9 @@ def _refactor_code_for_package(
         |__ test.py
     """
 
-    def _move_package_with_conflicting_name(target: Path, conflicting_name: str):
+    def _move_package_with_conflicting_name(
+        target: Path, conflicting_name: str
+    ) -> None:
         tmp_name = "tmp_name"
         tmp_module = target.parent / tmp_name
         _rename_package(project, target.as_posix(), tmp_name)
@@ -680,7 +820,7 @@ def _refactor_code_for_package(
 
     # Copy tests in appropriate folder structure
     if tests_path.exists():
-        tests_target = tests_path.relative_to(project_metadata.source_dir)
+        tests_target = tests_path.relative_to(project_metadata.project_path)
         full_path = _create_nested_package(project, tests_target)
         # overwrite=True to update the __init__.py files generated by create_package
         _sync_dirs(tests_path, full_path, overwrite=True)
@@ -707,17 +847,13 @@ def _refactor_code_for_package(
         _move_package_with_conflicting_name(tests_target, "tests")
 
 
-_SourcePathType = Union[Path, List[Tuple[Path, str]]]
-
-
-# pylint: disable=too-many-arguments,too-many-locals
-def _generate_sdist_file(
+def _generate_sdist_file(  # noqa: PLR0913,too-many-locals
     micropkg_name: str,
     destination: Path,
-    source_paths: Tuple[_SourcePathType, ...],
+    source_paths: tuple[Path, Path, list[tuple[Path, str]]],
     version: str,
     metadata: ProjectMetadata,
-    alias: str = None,
+    alias: str | None = None,
 ) -> None:
     package_name = alias or micropkg_name
     package_source, tests_source, conf_source = source_paths
@@ -727,20 +863,24 @@ def _generate_sdist_file(
 
         project = Project(temp_dir_path)  # project where to do refactoring
         _refactor_code_for_package(
-            project, package_source, tests_source, alias, metadata  # type: ignore
+            project,
+            package_source,
+            tests_source,
+            alias,
+            metadata,
         )
         project.close()
 
         # Copy & "refactor" config
         _, _, conf_target = _get_package_artifacts(temp_dir_path, package_name)
-        _sync_path_list(conf_source, conf_target)  # type: ignore
+        _sync_path_list(conf_source, conf_target)
         if conf_target.is_dir() and alias:
             _rename_files(conf_target, micropkg_name, alias)
 
-        # Build a setup.py on the fly
+        # Build a pyproject.toml on the fly
         try:
             install_requires = _make_install_requires(
-                package_source / "requirements.txt"  # type: ignore
+                package_source / "requirements.txt"
             )
         except Exception as exc:
             click.secho("FAILED", fg="red")
@@ -748,9 +888,7 @@ def _generate_sdist_file(
             raise KedroCliError(f"{cls.__module__}.{cls.__qualname__}: {exc}") from exc
 
         _generate_manifest_file(temp_dir_path)
-        setup_file = _generate_setup_file(
-            package_name, version, install_requires, temp_dir_path
-        )
+        _generate_pyproject_file(package_name, version, install_requires, temp_dir_path)
 
         package_file = destination / _get_sdist_name(name=package_name, version=version)
 
@@ -759,21 +897,21 @@ def _generate_sdist_file(
                 f"Package file {package_file} will be overwritten!", fg="yellow"
             )
 
-        # python setup.py sdist --formats=gztar --dist-dir <destination>
+        # python -m build --outdir <destination>
         call(
             [
                 sys.executable,
-                str(setup_file.resolve()),
-                "sdist",
-                "--formats=gztar",
-                "--dist-dir",
+                "-m",
+                "build",
+                "--sdist",
+                "--outdir",
                 str(destination),
             ],
             cwd=temp_dir,
         )
 
 
-def _generate_manifest_file(output_dir: Path):
+def _generate_manifest_file(output_dir: Path) -> None:
     manifest_file = output_dir / "MANIFEST.in"
     manifest_file.write_text(
         """
@@ -786,24 +924,24 @@ def _generate_manifest_file(output_dir: Path):
     )
 
 
-def _generate_setup_file(
-    package_name: str, version: str, install_requires: List[str], output_dir: Path
+def _generate_pyproject_file(
+    package_name: str, version: str, install_requires: list[str], output_dir: Path
 ) -> Path:
-    setup_file = output_dir / "setup.py"
+    pyproject_file = output_dir / "pyproject.toml"
 
-    setup_file_context = {
+    pyproject_file_context = {
         "name": package_name,
         "version": version,
         "install_requires": install_requires,
     }
 
-    setup_file.write_text(_SETUP_PY_TEMPLATE.format(**setup_file_context))
-    return setup_file
+    pyproject_file.write_text(_PYPROJECT_TOML_TEMPLATE.format(**pyproject_file_context))
+    return pyproject_file
 
 
 def _get_package_artifacts(
     source_path: Path, package_name: str
-) -> Tuple[Path, Path, Path]:
+) -> tuple[Path, Path, Path]:
     """From existing package, returns in order:
     source_path, tests_path, config_path
     """
@@ -817,7 +955,7 @@ def _get_package_artifacts(
 
 
 def _append_package_reqs(
-    requirements_txt: Path, package_reqs: List[str], package_name: str
+    requirements_txt: Path, package_reqs: list[str], package_name: str
 ) -> None:
     """Appends micro-package requirements to project level requirements.txt"""
     incoming_reqs = _safe_parse_requirements(package_reqs)
@@ -848,24 +986,44 @@ def _append_package_reqs(
             file.write(sep.join(sorted_reqs))
 
     click.secho(
-        "Use 'kedro build-reqs' to compile and 'pip install -r src/requirements.lock' to install "
-        "the updated list of requirements."
+        "Use 'pip-compile requirements.txt --output-file requirements.lock' to compile "
+        "and 'pip install -r requirements.lock' to install the updated list of requirements."
     )
 
 
+def _get_all_library_reqs(metadata: PackageMetadata) -> list[str]:
+    """Get all library requirements from metadata, leaving markers intact."""
+    # See https://discuss.python.org/t/\
+    # programmatically-getting-non-optional-requirements-of-current-directory/26963/2
+    return [
+        str(_EquivalentRequirement(dep_str))
+        for dep_str in metadata.get_all("Requires-Dist", [])
+    ]
+
+
 def _safe_parse_requirements(
-    requirements: Union[str, Iterable[str]]
-) -> Set[pkg_resources.Requirement]:
-    """Safely parse a requirement or set of requirements. This effectively replaces
-    pkg_resources.parse_requirements, which blows up with a ValueError as soon as it
+    requirements: str | Iterable[str],
+) -> set[_EquivalentRequirement]:
+    """Safely parse a requirement or set of requirements. This avoids blowing up when it
     encounters a requirement it cannot parse (e.g. `-r requirements.txt`). This way
     we can still extract all the parseable requirements out of a set containing some
     unparseable requirements.
     """
     parseable_requirements = set()
-    for requirement in pkg_resources.yield_lines(requirements):
-        try:
-            parseable_requirements.add(pkg_resources.Requirement.parse(requirement))
-        except ValueError:
-            continue
+    if isinstance(requirements, str):
+        requirements = requirements.splitlines()
+    # TODO: Properly handle continuation lines,
+    # see https://github.com/pypa/setuptools/blob/v67.8.0/setuptools/_reqs.py
+    for requirement_line in requirements:
+        if (
+            requirement_line
+            and not requirement_line.startswith("#")
+            and not requirement_line.startswith("-e")
+        ):
+            try:
+                parseable_requirements.add(
+                    _EquivalentRequirement(_drop_comment(requirement_line))
+                )
+            except InvalidRequirement:
+                continue
     return parseable_requirements
