@@ -1,8 +1,11 @@
 """This module provides an utility function to retrieve the global hook_manager singleton
 in a Kedro's execution process.
 """
+
 import logging
-from typing import Any, Iterable
+from collections.abc import Iterable
+from inspect import isclass
+from typing import Any
 
 from pluggy import PluginManager
 
@@ -23,6 +26,10 @@ logger = logging.getLogger(__name__)
 def _create_hook_manager() -> PluginManager:
     """Create a new PluginManager instance and register Kedro's hook specs."""
     manager = PluginManager(HOOK_NAMESPACE)
+    manager.trace.root.setwriter(
+        logger.debug if logger.getEffectiveLevel() == logging.DEBUG else None
+    )
+    manager.enable_tracing()
     manager.add_hookspecs(NodeSpecs)
     manager.add_hookspecs(PipelineSpecs)
     manager.add_hookspecs(DataCatalogSpecs)
@@ -44,13 +51,18 @@ def _register_hooks(hook_manager: PluginManager, hooks: Iterable[Any]) -> None:
         # case hooks have already been registered, so we perform a simple check
         # here to avoid an error being raised and break user's workflow.
         if not hook_manager.is_registered(hooks_collection):
+            if isclass(hooks_collection):
+                raise TypeError(
+                    "KedroSession expects hooks to be registered as instances. "
+                    "Have you forgotten the `()` when registering a hook class ?"
+                )
             hook_manager.register(hooks_collection)
 
 
-def _register_hooks_setuptools(
+def _register_hooks_entry_points(
     hook_manager: PluginManager, disabled_plugins: Iterable[str]
 ) -> None:
-    """Register pluggy hooks from setuptools entrypoints.
+    """Register pluggy hooks from python package entrypoints.
 
     Args:
         hook_manager: Hook manager instance to register the hooks with.
@@ -60,10 +72,13 @@ def _register_hooks_setuptools(
 
     """
     already_registered = hook_manager.get_plugins()
+    # Method name is misleading:
+    # entry points are standard and don't require setuptools,
+    # see https://packaging.python.org/en/latest/specifications/entry-points/
     hook_manager.load_setuptools_entrypoints(_PLUGIN_HOOKS)
     disabled_plugins = set(disabled_plugins)
 
-    # Get list of plugin/distinfo tuples for all setuptools registered plugins.
+    # Get list of plugin/distinfo tuples for all registered plugins.
     plugininfo = hook_manager.list_plugin_distinfo()
     plugin_names = set()
     disabled_plugin_names = set()
@@ -96,11 +111,11 @@ class _NullPluginManager:
     """This class creates an empty ``hook_manager`` that will ignore all calls to hooks,
     allowing the runner to function if no ``hook_manager`` has been instantiated."""
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         pass
 
-    def __getattr__(self, name):
+    def __getattr__(self, name: str) -> Any:
         return self
 
-    def __call__(self, *args, **kwargs):
+    def __call__(self, *args: Any, **kwargs: Any) -> None:
         pass
