@@ -2,28 +2,32 @@
 For more information about these specifications, please visit
 [Pluggy's documentation](https://pluggy.readthedocs.io/en/stable/#specs)
 """
-from typing import Any, Dict, Optional
 
-from kedro.framework.context import KedroContext
-from kedro.io import DataCatalog
-from kedro.pipeline import Pipeline
-from kedro.pipeline.node import Node
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Any
 
 from .markers import hook_spec
 
+if TYPE_CHECKING:
+    from kedro.framework.context import KedroContext
+    from kedro.io import CatalogProtocol
+    from kedro.pipeline import Pipeline
+    from kedro.pipeline.node import Node
+
 
 class DataCatalogSpecs:
     """Namespace that defines all specifications for a data catalog's lifecycle hooks."""
 
     @hook_spec
-    def after_catalog_created(  # pylint: disable=too-many-arguments
+    def after_catalog_created(  # noqa: PLR0913
         self,
-        catalog: DataCatalog,
-        conf_catalog: Dict[str, Any],
-        conf_creds: Dict[str, Any],
-        feed_dict: Dict[str, Any],
+        catalog: CatalogProtocol,
+        conf_catalog: dict[str, Any],
+        conf_creds: dict[str, Any],
+        feed_dict: dict[str, Any],
         save_version: str,
-        load_versions: Dict[str, str],
+        load_versions: dict[str, str],
     ) -> None:
         """Hooks to be invoked after a data catalog is created.
         It receives the ``catalog`` as well as
@@ -46,20 +50,20 @@ class NodeSpecs:
     """Namespace that defines all specifications for a node's lifecycle hooks."""
 
     @hook_spec
-    def before_node_run(  # pylint: disable=too-many-arguments
+    def before_node_run(
         self,
         node: Node,
-        catalog: DataCatalog,
-        inputs: Dict[str, Any],
+        catalog: CatalogProtocol,
+        inputs: dict[str, Any],
         is_async: bool,
         session_id: str,
-    ) -> Optional[Dict[str, Any]]:
+    ) -> dict[str, Any] | None:
         """Hook to be invoked before a node runs.
         The arguments received are the same as those used by ``kedro.runner.run_node``
 
         Args:
             node: The ``Node`` to run.
-            catalog: A ``DataCatalog`` containing the node's inputs and outputs.
+            catalog: An implemented instance of ``CatalogProtocol`` containing the node's inputs and outputs.
             inputs: The dictionary of inputs dataset.
                 The keys are dataset names and the values are the actual loaded input data,
                 not the dataset instance.
@@ -74,12 +78,12 @@ class NodeSpecs:
         pass
 
     @hook_spec
-    def after_node_run(  # pylint: disable=too-many-arguments
+    def after_node_run(  # noqa: PLR0913
         self,
         node: Node,
-        catalog: DataCatalog,
-        inputs: Dict[str, Any],
-        outputs: Dict[str, Any],
+        catalog: CatalogProtocol,
+        inputs: dict[str, Any],
+        outputs: dict[str, Any],
         is_async: bool,
         session_id: str,
     ) -> None:
@@ -89,7 +93,7 @@ class NodeSpecs:
 
         Args:
             node: The ``Node`` that ran.
-            catalog: A ``DataCatalog`` containing the node's inputs and outputs.
+            catalog: An implemented instance of ``CatalogProtocol`` containing the node's inputs and outputs.
             inputs: The dictionary of inputs dataset.
                 The keys are dataset names and the values are the actual loaded input data,
                 not the dataset instance.
@@ -102,15 +106,15 @@ class NodeSpecs:
         pass
 
     @hook_spec
-    def on_node_error(  # pylint: disable=too-many-arguments
+    def on_node_error(  # noqa: PLR0913
         self,
         error: Exception,
         node: Node,
-        catalog: DataCatalog,
-        inputs: Dict[str, Any],
+        catalog: CatalogProtocol,
+        inputs: dict[str, Any],
         is_async: bool,
         session_id: str,
-    ):
+    ) -> None:
         """Hook to be invoked if a node run throws an uncaught error.
         The signature of this error hook should match the signature of ``before_node_run``
         along with the error that was raised.
@@ -118,7 +122,7 @@ class NodeSpecs:
         Args:
             error: The uncaught exception thrown during the node run.
             node: The ``Node`` to run.
-            catalog: A ``DataCatalog`` containing the node's inputs and outputs.
+            catalog: An implemented instance of ``CatalogProtocol`` containing the node's inputs and outputs.
             inputs: The dictionary of inputs dataset.
                 The keys are dataset names and the values are the actual loaded input data,
                 not the dataset instance.
@@ -133,7 +137,7 @@ class PipelineSpecs:
 
     @hook_spec
     def before_pipeline_run(
-        self, run_params: Dict[str, Any], pipeline: Pipeline, catalog: DataCatalog
+        self, run_params: dict[str, Any], pipeline: Pipeline, catalog: CatalogProtocol
     ) -> None:
         """Hook to be invoked before a pipeline runs.
 
@@ -153,22 +157,24 @@ class PipelineSpecs:
                      "from_inputs": Optional[List[str]],
                      "to_outputs": Optional[List[str]],
                      "load_versions": Optional[List[str]],
-                     "pipeline_name": str,
                      "extra_params": Optional[Dict[str, Any]]
+                     "pipeline_name": str,
+                     "namespace": Optional[str],
+                     "runner": str,
                    }
 
             pipeline: The ``Pipeline`` that will be run.
-            catalog: The ``DataCatalog`` to be used during the run.
+            catalog: An implemented instance of ``CatalogProtocol`` to be used during the run.
         """
         pass
 
     @hook_spec
     def after_pipeline_run(
         self,
-        run_params: Dict[str, Any],
-        run_result: Dict[str, Any],
+        run_params: dict[str, Any],
+        run_result: dict[str, Any],
         pipeline: Pipeline,
-        catalog: DataCatalog,
+        catalog: CatalogProtocol,
     ) -> None:
         """Hook to be invoked after a pipeline runs.
 
@@ -188,13 +194,15 @@ class PipelineSpecs:
                      "from_inputs": Optional[List[str]],
                      "to_outputs": Optional[List[str]],
                      "load_versions": Optional[List[str]],
-                     "pipeline_name": str,
                      "extra_params": Optional[Dict[str, Any]]
+                     "pipeline_name": str,
+                     "namespace": Optional[str],
+                     "runner": str,
                    }
 
             run_result: The output of ``Pipeline`` run.
             pipeline: The ``Pipeline`` that was run.
-            catalog: The ``DataCatalog`` used during the run.
+            catalog: An implemented instance of ``CatalogProtocol`` used during the run.
         """
         pass
 
@@ -202,10 +210,10 @@ class PipelineSpecs:
     def on_pipeline_error(
         self,
         error: Exception,
-        run_params: Dict[str, Any],
+        run_params: dict[str, Any],
         pipeline: Pipeline,
-        catalog: DataCatalog,
-    ):
+        catalog: CatalogProtocol,
+    ) -> None:
         """Hook to be invoked if a pipeline run throws an uncaught Exception.
         The signature of this error hook should match the signature of ``before_pipeline_run``
         along with the error that was raised.
@@ -227,11 +235,14 @@ class PipelineSpecs:
                      "from_inputs": Optional[List[str]],
                      "to_outputs": Optional[List[str]],
                      "load_versions": Optional[List[str]],
-                     "pipeline_name": str,
                      "extra_params": Optional[Dict[str, Any]]
+                     "pipeline_name": str,
+                     "namespace": Optional[str],
+                     "runner": str,
                    }
+
             pipeline: The ``Pipeline`` that will was run.
-            catalog: The ``DataCatalog`` used during the run.
+            catalog: An implemented instance of ``CatalogProtocol`` used during the run.
         """
         pass
 
