@@ -1,39 +1,42 @@
-"""``kedro.framework.project`` module provides utitlity to
+"""``kedro.framework.project`` module provides utility to
 configure a Kedro project and access its settings."""
-# pylint: disable=redefined-outer-name,unused-argument,global-statement
+
+from __future__ import annotations
+
 import importlib
 import logging.config
 import operator
 import os
-import sys
 import traceback
-import types
 import warnings
 from collections import UserDict
 from collections.abc import MutableMapping
 from pathlib import Path
-from typing import Any, Dict, Optional
+from typing import TYPE_CHECKING, Any
 
-import click
+import dynaconf
 import importlib_resources
-import rich.pretty
-import rich.traceback
 import yaml
 from dynaconf import LazySettings
 from dynaconf.validator import ValidationError, Validator
 
+from kedro.io import CatalogProtocol
+from kedro.io.warning_utils import suppress_catalog_warning
 from kedro.pipeline import Pipeline, pipeline
 
+if TYPE_CHECKING:
+    import types
+
 IMPORT_ERROR_MESSAGE = (
     "An error occurred while importing the '{module}' module. Nothing "
     "defined therein will be returned by 'find_pipelines'.\n\n{tb_exc}"
 )
 
 
-def _get_default_class(class_import_path):
+def _get_default_class(class_import_path: str) -> Any:
     module, _, class_name = class_import_path.rpartition(".")
 
-    def validator_func(settings, validators):
+    def validator_func(settings: dynaconf.base.Settings, validators: Any) -> Any:
         return getattr(importlib.import_module(module), class_name)
 
     return validator_func
@@ -42,7 +45,9 @@ def _get_default_class(class_import_path):
 class _IsSubclassValidator(Validator):
     """A validator to check if the supplied setting value is a subclass of the default class"""
 
-    def validate(self, settings, *args, **kwargs):
+    def validate(
+        self, settings: dynaconf.base.Settings, *args: Any, **kwargs: Any
+    ) -> None:
         super().validate(settings, *args, **kwargs)
 
         default_class = self.default(settings, self)
@@ -56,11 +61,33 @@ class _IsSubclassValidator(Validator):
                 )
 
 
+class _ImplementsCatalogProtocolValidator(Validator):
+    """A validator to check if the supplied setting value is a subclass of the default class"""
+
+    def validate(
+        self, settings: dynaconf.base.Settings, *args: Any, **kwargs: Any
+    ) -> None:
+        super().validate(settings, *args, **kwargs)
+
+        protocol = CatalogProtocol
+        for name in self.names:
+            setting_value = getattr(settings, name)
+            with suppress_catalog_warning():
+                if not isinstance(setting_value(), protocol):
+                    raise ValidationError(
+                        f"Invalid value '{setting_value.__module__}.{setting_value.__qualname__}' "
+                        f"received for setting '{name}'. It must implement "
+                        f"'{protocol.__module__}.{protocol.__qualname__}'."
+                    )
+
+
 class _HasSharedParentClassValidator(Validator):
     """A validator to check that the parent of the default class is an ancestor of
     the settings value."""
 
-    def validate(self, settings, *args, **kwargs):
+    def validate(
+        self, settings: dynaconf.base.Settings, *args: Any, **kwargs: Any
+    ) -> None:
         super().validate(settings, *args, **kwargs)
 
         default_class = self.default(settings, self)
@@ -99,20 +126,23 @@ class _ProjectSettings(LazySettings):
     )
     _SESSION_STORE_CLASS = _IsSubclassValidator(
         "SESSION_STORE_CLASS",
-        default=_get_default_class("kedro.framework.session.session.BaseSessionStore"),
+        default=_get_default_class("kedro.framework.session.store.BaseSessionStore"),
     )
     _SESSION_STORE_ARGS = Validator("SESSION_STORE_ARGS", default={})
     _DISABLE_HOOKS_FOR_PLUGINS = Validator("DISABLE_HOOKS_FOR_PLUGINS", default=tuple())
     _CONFIG_LOADER_CLASS = _HasSharedParentClassValidator(
-        "CONFIG_LOADER_CLASS", default=_get_default_class("kedro.config.ConfigLoader")
+        "CONFIG_LOADER_CLASS",
+        default=_get_default_class("kedro.config.OmegaConfigLoader"),
     )
-    _CONFIG_LOADER_ARGS = Validator("CONFIG_LOADER_ARGS", default={})
-    _DATA_CATALOG_CLASS = _IsSubclassValidator(
-        "DATA_CATALOG_CLASS", default=_get_default_class("kedro.io.DataCatalog")
+    _CONFIG_LOADER_ARGS = Validator(
+        "CONFIG_LOADER_ARGS", default={"base_env": "base", "default_run_env": "local"}
+    )
+    _DATA_CATALOG_CLASS = _ImplementsCatalogProtocolValidator(
+        "DATA_CATALOG_CLASS",
+        default=_get_default_class("kedro.io.DataCatalog"),
     )
 
-    def __init__(self, *args, **kwargs):
-
+    def __init__(self, *args: Any, **kwargs: Any):
         kwargs.update(
             validators=[
                 self._CONF_SOURCE,
@@ -129,12 +159,12 @@ class _ProjectSettings(LazySettings):
         super().__init__(*args, **kwargs)
 
 
-def _load_data_wrapper(func):
+def _load_data_wrapper(func: Any) -> Any:
     """Wrap a method in _ProjectPipelines so that data is loaded on first access.
     Taking inspiration from dynaconf.utils.functional.new_method_proxy
     """
-    # pylint: disable=protected-access
-    def inner(self, *args, **kwargs):
+
+    def inner(self: Any, *args: Any, **kwargs: Any) -> Any:
         self._load_data()
         return func(self._content, *args, **kwargs)
 
@@ -159,17 +189,17 @@ class _ProjectPipelines(MutableMapping):
     """
 
     def __init__(self) -> None:
-        self._pipelines_module: Optional[str] = None
+        self._pipelines_module: str | None = None
         self._is_data_loaded = False
-        self._content: Dict[str, Pipeline] = {}
+        self._content: dict[str, Pipeline] = {}
 
     @staticmethod
-    def _get_pipelines_registry_callable(pipelines_module: str):
+    def _get_pipelines_registry_callable(pipelines_module: str) -> Any:
         module_obj = importlib.import_module(pipelines_module)
         register_pipelines = getattr(module_obj, "register_pipelines")
         return register_pipelines
 
-    def _load_data(self):
+    def _load_data(self) -> None:
         """Lazily read pipelines defined in the pipelines registry module."""
 
         # If the pipelines dictionary has not been configured with a pipelines module
@@ -185,7 +215,7 @@ class _ProjectPipelines(MutableMapping):
         self._content = project_pipelines
         self._is_data_loaded = True
 
-    def configure(self, pipelines_module: Optional[str] = None) -> None:
+    def configure(self, pipelines_module: str | None = None) -> None:
         """Configure the pipelines_module to load the pipelines dictionary.
         Reset the data loading state so that after every ``configure`` call,
         data are reloaded.
@@ -210,28 +240,38 @@ class _ProjectPipelines(MutableMapping):
 
 
 class _ProjectLogging(UserDict):
-    # pylint: disable=super-init-not-called
-    def __init__(self):
-        """Initialise project logging with default configuration. Also enable
-        rich tracebacks."""
-        default_logging = (Path(__file__).parent / "default_logging.yml").read_text(
-            encoding="utf-8"
+    def __init__(self) -> None:
+        """Initialise project logging. The path to logging configuration is given in
+        environment variable KEDRO_LOGGING_CONFIG (defaults to conf/logging.yml)."""
+        logger = logging.getLogger(__name__)
+        user_logging_path = os.environ.get("KEDRO_LOGGING_CONFIG")
+        project_logging_path = Path("conf/logging.yml")
+        default_logging_path = Path(
+            Path(__file__).parent / "rich_logging.yml"
+            if importlib.util.find_spec("rich")
+            else Path(__file__).parent / "default_logging.yml",
         )
-        self.configure(yaml.safe_load(default_logging))
-        logging.captureWarnings(True)
+        path: str | Path
+        msg = ""
 
-        # We suppress click here to hide tracebacks related to it conversely,
-        # kedro is not suppressed to show its tracebacks for easier debugging.
-        # sys.executable is used to get the kedro executable path to hide the
-        # top level traceback.
-        # Rich traceback handling does not work on databricks. Hopefully this will be
-        # fixed on their side at some point, but until then we disable it.
-        # See https://github.com/Textualize/rich/issues/2455
-        if "DATABRICKS_RUNTIME_VERSION" not in os.environ:
-            rich.traceback.install(suppress=[click, str(Path(sys.executable).parent)])
-        rich.pretty.install()
+        if user_logging_path:
+            path = user_logging_path
 
-    def configure(self, logging_config: Dict[str, Any]) -> None:
+        elif project_logging_path.exists():
+            path = project_logging_path
+            msg = "You can change this by setting the KEDRO_LOGGING_CONFIG environment variable accordingly."
+        else:
+            # Fallback to the framework default loggings
+            path = default_logging_path
+
+        msg = f"Using '{path!s}' as logging configuration. " + msg
+
+        # Load and apply the logging configuration
+        logging_config = Path(path).read_text(encoding="utf-8")
+        self.configure(yaml.safe_load(logging_config))
+        logger.info(msg)
+
+    def configure(self, logging_config: dict[str, Any]) -> None:
         """Configure project logging using ``logging_config`` (e.g. from project
         logging.yml). We store this in the UserDict data so that it can be reconfigured
         in _bootstrap_subprocess.
@@ -239,6 +279,14 @@ class _ProjectLogging(UserDict):
         logging.config.dictConfig(logging_config)
         self.data = logging_config
 
+    def set_project_logging(self, package_name: str) -> None:
+        """Add the project level logging to the loggers upon provision of a package name.
+        Checks if project logger already exists to prevent overwriting, if none exists
+        it defaults to setting project logs at INFO level."""
+        if package_name not in self.data["loggers"]:
+            self.data["loggers"][package_name] = {"level": "INFO"}
+            self.configure(self.data)
+
 
 PACKAGE_NAME = None
 LOGGING = _ProjectLogging()
@@ -248,7 +296,7 @@ settings = _ProjectSettings()
 pipelines = _ProjectPipelines()
 
 
-def configure_project(package_name: str):
+def configure_project(package_name: str) -> None:
     """Configure a Kedro project by populating its settings with values
     defined in user's settings.py and pipeline_registry.py.
     """
@@ -262,22 +310,25 @@ def configure_project(package_name: str):
     # global variable to make it easily accessible. This is used by validate_settings()
     # below, and also by ParallelRunner on Windows, as package_name is required every
     # time a new subprocess is spawned.
-    global PACKAGE_NAME
+    global PACKAGE_NAME  # noqa: PLW0603
     PACKAGE_NAME = package_name
 
+    if PACKAGE_NAME:
+        LOGGING.set_project_logging(PACKAGE_NAME)
 
-def configure_logging(logging_config: Dict[str, Any]) -> None:
+
+def configure_logging(logging_config: dict[str, Any]) -> None:
     """Configure logging according to ``logging_config`` dictionary."""
     LOGGING.configure(logging_config)
 
 
-def validate_settings():
-    """Eagerly validate that the settings module is importable. This is desirable to
+def validate_settings() -> None:
+    """Eagerly validate that the settings module is importable if it exists. This is desirable to
     surface any syntax or import errors early. In particular, without eagerly importing
     the settings module, dynaconf would silence any import error (e.g. missing
     dependency, missing/mislabelled pipeline), and users would instead get a cryptic
     error message ``Expected an instance of `ConfigLoader`, got `NoneType` instead``.
-    More info on the dynaconf issue: https://github.com/rochacbruno/dynaconf/issues/460
+    More info on the dynaconf issue: https://github.com/dynaconf/dynaconf/issues/460
     """
     if PACKAGE_NAME is None:
         raise ValueError(
@@ -285,11 +336,15 @@ def validate_settings():
             "'bootstrap_project'. This should happen automatically if you are using "
             "Kedro command line interface."
         )
-
-    importlib.import_module(f"{PACKAGE_NAME}.settings")
+    # Check if file exists, if it does, validate it.
+    if importlib.util.find_spec(f"{PACKAGE_NAME}.settings") is not None:
+        importlib.import_module(f"{PACKAGE_NAME}.settings")
+    else:
+        logger = logging.getLogger(__name__)
+        logger.warning("No 'settings.py' found, defaults will be used.")
 
 
-def _create_pipeline(pipeline_module: types.ModuleType) -> Optional[Pipeline]:
+def _create_pipeline(pipeline_module: types.ModuleType) -> Pipeline | None:
     if not hasattr(pipeline_module, "create_pipeline"):
         warnings.warn(
             f"The '{pipeline_module.__name__}' module does not "
@@ -312,7 +367,7 @@ def _create_pipeline(pipeline_module: types.ModuleType) -> Optional[Pipeline]:
     return obj
 
 
-def find_pipelines() -> Dict[str, Pipeline]:
+def find_pipelines(raise_errors: bool = False) -> dict[str, Pipeline]:  # noqa: PLR0912
     """Automatically find modular pipelines having a ``create_pipeline``
     function. By default, projects created using Kedro 0.18.3 and higher
     call this function to autoregister pipelines upon creation/addition.
@@ -322,15 +377,25 @@ def find_pipelines() -> Dict[str, Pipeline]:
     can modify the mapping generated by the ``find_pipelines`` function.
 
     For more information on the pipeline registry and autodiscovery, see
-    https://kedro.readthedocs.io/en/latest/nodes_and_pipelines/pipeline_registry.html
+    https://docs.kedro.org/en/stable/nodes_and_pipelines/pipeline_registry.html
+
+    Args:
+        raise_errors: If ``True``, raise an error upon failed discovery.
 
     Returns:
         A generated mapping from pipeline names to ``Pipeline`` objects.
 
+    Raises:
+        ImportError: When a module does not expose a ``create_pipeline``
+            function, the ``create_pipeline`` function does not return a
+            ``Pipeline`` object, or if the module import fails up front.
+            If ``raise_errors`` is ``False``, see Warns section instead.
+
     Warns:
         UserWarning: When a module does not expose a ``create_pipeline``
             function, the ``create_pipeline`` function does not return a
             ``Pipeline`` object, or if the module import fails up front.
+            If ``raise_errors`` is ``True``, see Raises section instead.
     """
     pipeline_obj = None
 
@@ -338,8 +403,14 @@ def find_pipelines() -> Dict[str, Pipeline]:
     pipeline_module_name = f"{PACKAGE_NAME}.pipeline"
     try:
         pipeline_module = importlib.import_module(pipeline_module_name)
-    except Exception as exc:  # pylint: disable=broad-except
+    except Exception as exc:
         if str(exc) != f"No module named '{pipeline_module_name}'":
+            if raise_errors:
+                raise ImportError(
+                    f"An error occurred while importing the "
+                    f"'{pipeline_module_name}' module."
+                ) from exc
+
             warnings.warn(
                 IMPORT_ERROR_MESSAGE.format(
                     module=pipeline_module_name, tb_exc=traceback.format_exc()
@@ -364,11 +435,20 @@ def find_pipelines() -> Dict[str, Pipeline]:
         pipeline_name = pipeline_dir.name
         if pipeline_name == "__pycache__":
             continue
+        # Prevent imports of hidden directories/files
+        if pipeline_name.startswith("."):
+            continue
 
         pipeline_module_name = f"{PACKAGE_NAME}.pipelines.{pipeline_name}"
         try:
             pipeline_module = importlib.import_module(pipeline_module_name)
-        except:  # pylint: disable=bare-except  # noqa: E722
+        except Exception as exc:
+            if raise_errors:
+                raise ImportError(
+                    f"An error occurred while importing the "
+                    f"'{pipeline_module_name}' module."
+                ) from exc
+
             warnings.warn(
                 IMPORT_ERROR_MESSAGE.format(
                     module=pipeline_module_name, tb_exc=traceback.format_exc()
