@@ -1,7 +1,10 @@
 """This module implements a dict-like store object used to persist Kedro sessions."""
+
+from __future__ import annotations
+
 import logging
 from collections import UserDict
-from typing import Any, Dict
+from typing import Any
 
 
 class BaseSessionStore(UserDict):
@@ -19,7 +22,7 @@ class BaseSessionStore(UserDict):
     def _logger(self) -> logging.Logger:
         return logging.getLogger(__name__)
 
-    def read(self) -> Dict[str, Any]:
+    def read(self) -> dict[str, Any]:
         """Read the data from the session store.
 
         Returns:
@@ -31,7 +34,7 @@ class BaseSessionStore(UserDict):
         )
         return {}
 
-    def save(self):
+    def save(self) -> None:
         """Persist the session store"""
         self._logger.debug(
             "'save()' not implemented for '%s'. Skipping the step.",
