@@ -1,36 +1,41 @@
-"""``LambdaDataSet`` is an implementation of ``AbstractDataSet`` which allows for
+"""``LambdaDataset`` is an implementation of ``AbstractDataset`` which allows for
 providing custom load, save, and exists methods without extending
-``AbstractDataSet``.
+``AbstractDataset``.
 """
-from typing import Any, Callable, Dict, Optional
 
-from kedro.io.core import AbstractDataSet, DataSetError
+from __future__ import annotations
+
+import warnings
+from typing import Any, Callable
+
+from kedro import KedroDeprecationWarning
+from kedro.io.core import AbstractDataset, DatasetError
 
 
-class LambdaDataSet(AbstractDataSet):
-    """``LambdaDataSet`` loads and saves data to a data set.
+class LambdaDataset(AbstractDataset):
+    """``LambdaDataset`` loads and saves data to a dataset.
     It relies on delegating to specific implementation such as csv, sql, etc.
 
-    ``LambdaDataSet`` class captures Exceptions while performing operations on
-    composed ``DataSet`` implementations. The composed data set is
+    ``LambdaDataset`` class captures Exceptions while performing operations on
+    composed ``Dataset`` implementations. The composed dataset is
     responsible for providing information on how to resolve the issue when
     possible. This information should be available through str(error).
 
     Example:
     ::
 
-        >>> from kedro.io import LambdaDataSet
+        >>> from kedro.io import LambdaDataset
         >>> import pandas as pd
         >>>
         >>> file_name = "test.csv"
         >>> def load() -> pd.DataFrame:
         >>>     raise FileNotFoundError("'{}' csv file not found."
         >>>                             .format(file_name))
-        >>> data_set = LambdaDataSet(load, None)
+        >>> dataset = LambdaDataset(load, None)
     """
 
-    def _describe(self) -> Dict[str, Any]:
-        def _to_str(func):
+    def _describe(self) -> dict[str, Any]:
+        def _to_str(func: Any) -> str | None:
             if not func:
                 return None
             try:
@@ -47,22 +52,22 @@ class LambdaDataSet(AbstractDataSet):
 
         return descr
 
-    def _save(self, data: Any) -> None:
-        if not self.__save:
-            raise DataSetError(
-                "Cannot save to data set. No 'save' function "
-                "provided when LambdaDataSet was created."
-            )
-        self.__save(data)
-
     def _load(self) -> Any:
         if not self.__load:
-            raise DataSetError(
-                "Cannot load data set. No 'load' function "
-                "provided when LambdaDataSet was created."
+            raise DatasetError(
+                "Cannot load dataset. No 'load' function "
+                "provided when LambdaDataset was created."
             )
         return self.__load()
 
+    def _save(self, data: Any) -> None:
+        if not self.__save:
+            raise DatasetError(
+                "Cannot save to dataset. No 'save' function "
+                "provided when LambdaDataset was created."
+            )
+        self.__save(data)
+
     def _exists(self) -> bool:
         if not self.__exists:
             return super()._exists()
@@ -76,25 +81,33 @@ class LambdaDataSet(AbstractDataSet):
 
     def __init__(
         self,
-        load: Optional[Callable[[], Any]],
-        save: Optional[Callable[[Any], None]],
-        exists: Callable[[], bool] = None,
-        release: Callable[[], None] = None,
+        load: Callable[[], Any] | None,
+        save: Callable[[Any], None] | None,
+        exists: Callable[[], bool] | None = None,
+        release: Callable[[], None] | None = None,
+        metadata: dict[str, Any] | None = None,
     ):
-        """Creates a new instance of ``LambdaDataSet`` with references to the
-        required input/output data set methods.
+        """Creates a new instance of ``LambdaDataset`` with references to the
+        required input/output dataset methods.
 
         Args:
-            load: Method to load data from a data set.
-            save: Method to save data to a data set.
+            load: Method to load data from a dataset.
+            save: Method to save data to a dataset.
             exists: Method to check whether output data already exists.
             release: Method to release any cached information.
+            metadata: Any arbitrary metadata.
+                This is ignored by Kedro, but may be consumed by users or external plugins.
 
         Raises:
-            DataSetError: If a method is specified, but is not a Callable.
+            DatasetError: If a method is specified, but is not a Callable.
 
         """
 
+        warnings.warn(
+            "`LambdaDataset` has been deprecated and will be removed in Kedro 1.0.0.",
+            KedroDeprecationWarning,
+        )
+
         for name, value in [
             ("load", load),
             ("save", save),
@@ -102,8 +115,8 @@ class LambdaDataSet(AbstractDataSet):
             ("release", release),
         ]:
             if value is not None and not callable(value):
-                raise DataSetError(
-                    f"'{name}' function for LambdaDataSet must be a Callable. "
+                raise DatasetError(
+                    f"'{name}' function for LambdaDataset must be a Callable. "
                     f"Object of type '{value.__class__.__name__}' provided instead."
                 )
 
@@ -111,3 +124,4 @@ class LambdaDataSet(AbstractDataSet):
         self.__save = save
         self.__exists = exists
         self.__release = release
+        self.metadata = metadata
