@@ -1,40 +1,47 @@
-"""``MemoryDataSet`` is a data set implementation which handles in-memory data.
-"""
+"""``MemoryDataset`` is a dataset implementation which handles in-memory data."""
+
+from __future__ import annotations
 
 import copy
-from typing import Any, Dict
+from typing import Any
 
-from kedro.io.core import AbstractDataSet, DataSetError
+from kedro.io.core import AbstractDataset, DatasetError
 
 _EMPTY = object()
 
 
-class MemoryDataSet(AbstractDataSet):
-    """``MemoryDataSet`` loads and saves data from/to an in-memory
-    Python object.
+class MemoryDataset(AbstractDataset):
+    """``MemoryDataset`` loads and saves data from/to an in-memory
+    Python object. The `_EPHEMERAL` attribute is set to True to
+    indicate MemoryDataset's non-persistence.
 
     Example:
     ::
 
-        >>> from kedro.io import MemoryDataSet
+        >>> from kedro.io import MemoryDataset
         >>> import pandas as pd
         >>>
         >>> data = pd.DataFrame({'col1': [1, 2], 'col2': [4, 5],
         >>>                      'col3': [5, 6]})
-        >>> data_set = MemoryDataSet(data=data)
+        >>> dataset = MemoryDataset(data=data)
         >>>
-        >>> loaded_data = data_set.load()
+        >>> loaded_data = dataset.load()
         >>> assert loaded_data.equals(data)
         >>>
         >>> new_data = pd.DataFrame({'col1': [1, 2], 'col2': [4, 5]})
-        >>> data_set.save(new_data)
-        >>> reloaded_data = data_set.load()
+        >>> dataset.save(new_data)
+        >>> reloaded_data = dataset.load()
         >>> assert reloaded_data.equals(new_data)
 
     """
 
-    def __init__(self, data: Any = _EMPTY, copy_mode: str = None):
-        """Creates a new instance of ``MemoryDataSet`` pointing to the
+    def __init__(
+        self,
+        data: Any = _EMPTY,
+        copy_mode: str | None = None,
+        metadata: dict[str, Any] | None = None,
+    ):
+        """Creates a new instance of ``MemoryDataset`` pointing to the
         provided Python object.
 
         Args:
@@ -42,21 +49,25 @@ class MemoryDataSet(AbstractDataSet):
             copy_mode: The copy mode used to copy the data. Possible
                 values are: "deepcopy", "copy" and "assign". If not
                 provided, it is inferred based on the data type.
+            metadata: Any arbitrary metadata.
+                This is ignored by Kedro, but may be consumed by users or external plugins.
         """
         self._data = _EMPTY
         self._copy_mode = copy_mode
+        self.metadata = metadata
+        self._EPHEMERAL = True
         if data is not _EMPTY:
-            self._save(data)
+            self.save.__wrapped__(self, data)  # type: ignore[attr-defined]
 
-    def _load(self) -> Any:
+    def load(self) -> Any:
         if self._data is _EMPTY:
-            raise DataSetError("Data for MemoryDataSet has not been saved yet.")
+            raise DatasetError("Data for MemoryDataset has not been saved yet.")
 
         copy_mode = self._copy_mode or _infer_copy_mode(self._data)
         data = _copy_with_mode(self._data, copy_mode=copy_mode)
         return data
 
-    def _save(self, data: Any):
+    def save(self, data: Any) -> None:
         copy_mode = self._copy_mode or _infer_copy_mode(data)
         self._data = _copy_with_mode(data, copy_mode=copy_mode)
 
@@ -66,7 +77,7 @@ class MemoryDataSet(AbstractDataSet):
     def _release(self) -> None:
         self._data = _EMPTY
 
-    def _describe(self) -> Dict[str, Any]:
+    def _describe(self) -> dict[str, Any]:
         if self._data is not _EMPTY:
             return {"data": f"<{type(self._data).__name__}>"}
         # the string representation of datasets leaves out __init__
@@ -83,19 +94,22 @@ def _infer_copy_mode(data: Any) -> str:
     Returns:
         One of "copy", "assign" or "deepcopy" as the copy mode to use.
     """
-    # pylint: disable=import-outside-toplevel
     try:
         import pandas as pd
     except ImportError:  # pragma: no cover
-        pd = None  # pragma: no cover
+        pd = None  # type: ignore[assignment]  # pragma: no cover
     try:
         import numpy as np
     except ImportError:  # pragma: no cover
-        np = None  # pragma: no cover
+        np = None  # type: ignore[assignment] # pragma: no cover
+    try:
+        import ibis
+    except ImportError:  # pragma: no cover
+        ibis = None  # tpye: ignore[assignment] # pragma: no cover
 
     if pd and isinstance(data, pd.DataFrame) or np and isinstance(data, np.ndarray):
         copy_mode = "copy"
-    elif type(data).__name__ == "DataFrame":
+    elif type(data).__name__ == "DataFrame" or ibis and isinstance(data, ibis.Table):
         copy_mode = "assign"
     else:
         copy_mode = "deepcopy"
@@ -111,7 +125,7 @@ def _copy_with_mode(data: Any, copy_mode: str) -> Any:
         copy_mode: The copy mode to use, one of "deepcopy", "copy" and "assign".
 
     Raises:
-        DataSetError: If copy_mode is specified, but isn't valid
+        DatasetError: If copy_mode is specified, but isn't valid
             (i.e: not one of deepcopy, copy, assign)
 
     Returns:
@@ -124,9 +138,19 @@ def _copy_with_mode(data: Any, copy_mode: str) -> Any:
     elif copy_mode == "assign":
         copied_data = data
     else:
-        raise DataSetError(
+        raise DatasetError(
             f"Invalid copy mode: {copy_mode}. "
             f"Possible values are: deepcopy, copy, assign."
         )
 
     return copied_data
+
+
+def _is_memory_dataset(ds_or_type: AbstractDataset | str) -> bool:
+    """Check if dataset or str type provided is a MemoryDataset."""
+    if isinstance(ds_or_type, MemoryDataset):
+        return True
+    if isinstance(ds_or_type, str):
+        return ds_or_type in {"MemoryDataset", "kedro.io.memory_dataset.MemoryDataset"}
+
+    return False
