@@ -5,7 +5,6 @@ import pytest
 from kedro.utils import load_obj
 
 
-# pylint: disable=too-few-public-methods
 class DummyClass:
     pass
 
@@ -19,12 +18,6 @@ class TestExtractObject:
         extracted_obj = load_obj("DummyClass", "tests.test_utils")
         assert extracted_obj is DummyClass
 
-    def test_load_obj_invalid_attribute(self):
-        with pytest.raises(
-            AttributeError, match=r"Object 'InvalidClass' cannot be loaded"
-        ):
-            load_obj("InvalidClass", "tests.test_utils")
-
     def test_load_obj_invalid_module(self):
         with pytest.raises(ImportError, match=r"No module named 'missing_path'"):
             load_obj("InvalidClass", "missing_path")
